new;
/* ******************************************************************
*  This program provides estimates presented in KSS (2017). 
*  The program performs the following estimators 
*
*  1. 2 WAY FIXED EFFECT: country pair and time FE
*  2. 3D CCE G: 3DCCE with global factors only 
*  3. 3D CCE L: 3DCCE with local factors only
*  4. 3D CCE GL: 3DCCE with global and local factors  
*
*  This program also performs the CSD test as presented in KMSS(2017) and the alpha exponent test as presented in BKP (2015):
*  This program consists of the following steps:

	STEP 1:  loading data and necessary procedure ".src" files
	STEP 2: selecting the relevant dataset with no missing values
	STEP 3: estimation and CSD test of the 4 above mentioned estimators

*  Results are saved in excel files
*			
* ************************************************************************************** */


cls; _newb=1;

// ******************************************************
// STEP 1:  loading data and necessary procedure ".src" files
// ******************************************************

@ including the relevant procedure files @
#include C:\Yongcheol\Triple_index\empirics\KSS\index4kssp.src;
#include C:\Yongcheol\Triple_index\empirics\KSS\code_ord_holm.src;

@ loading the names and data files @
load names[15048,2]=C:\Yongcheol\Triple_index\empirics\KSS\nameslsm.txt;

data = xlsreadm("datakss","C2:AS15049",1,""); 


// ******************************************************
// STEP 2: selecting the relevant dataset with no missing values
// ******************************************************

@ variables selection@
@ border language distance@
Z = data[., 18 19]~ln(data[.,26]);

data[.,2:17]=repl2(data[.,2:17],0,miss(1,1));

@ selection of regressors@
@ year export gdph gdpf sim rlf rer euro cee @

data=data[.,1 9 12 13 30 31 34 37 40];

@ selection of the dependent variable @
data[., 2:4]=ln(data[.,2:4]);


@ selection of relevant pairs with no missing values @
pairs=getdist(names);

goodstr="Australia"|"Austria"|"BelLux"|"Canada"|"Denmark"|"Finland"|"France"|"Germany"|"Greece"|"Ireland"|"Italy"|"Japan"|"Netherlands"|"Norway"|"Portugal"|"Spain"|"Sweden"|"UnitedKingdom"|"UnitedStates"; 
goodpairs=selif(pairs,(sumc((pairs[.,1].==goodstr')').>0).and(sumc((pairs[.,2].==goodstr')').>0));

years=seqa(1970,1,44);
fullgoodpairs={};
i=1;
do while i<= rows(goodpairs);
  dd=data[getrows(names,goodpairs[i,.]),.];
  dd=selif(dd,dd[.,1].>1969);
  if (dd[.,1]==years)and(sumc(sumc(dd))./=miss(1,1));
    fullgoodpairs=fullgoodpairs|goodpairs[i,.];    
  endif;  
  i=i+1;
endo;    

@ generate the x matrix with no missing @
x=zeros((cols(data)-1)*44,rows(fullgoodpairs));
counter=1;
i=1;
do while i<=rows(goodpairs);
  dd=data[getrows(names,goodpairs[i,.]),.];
  dd=selif(dd,dd[.,1].>1969);
  if (dd[.,1]==years)and(sumc(sumc(dd))./=miss(1,1));
    x[.,counter]=reshape(dd[.,2:cols(dd)]',rows(dd)*(cols(dd)-1),1);
    counter=counter+1;
  endif;
  i=i+1;
endo;    

@ definition of useful parameters  @

ivar=getdist(fullgoodpairs[.,1]);           @relevant home countries@
jvar=getdist(sortcc(fullgoodpairs[.,2],1)); @relevant destination countries@
goodstr=selif(goodstr,sumc(goodstr'.==ivar).>0);  
ni=repl(fullgoodpairs[.,1],goodstr); @index for relevant home countries@
nj=repl(fullgoodpairs[.,2],goodstr); @index for relevant home countries@


@selection of dependent and independent@

y=x[1:44,.];        @ t by n@
x=x[45:rows(x),.];  @ t*k by n@

// ***********************************************************************
// STEP 3: estimation and CSD tests for 2WAYFE; 3DCCE_G; 3DCCE_L; 3DCCE_GL
// ***********************************************************************


@ estimates of 2 WAY FE @

{bp11,bmg11,vbp11,vbmg11,tbp11,tbmg11,bres11,bsep11,bsemg11}=index3mis(y,x,1,2,zeros(cols(data)-2,1),rows(ivar),ni,nj,5,7);

Xname = "gdph"|"gdpf"|"sim"|"rlf"|"rer"|"euro"|"cee";

PROUT = Xname~bp11~bsep11~tbp11;
"BETA TWO WAY FE ";"  regressors   bpool    s.e.     t-stat  ";

call printfm(PRout,0~1~1~1,"s"~8~8|"lf"~10~4|"lf"~10~4|"lf"~10~4); 
"";

@ output in excel 2 WAY POOL FE  @
ret = xlsWritem(PROUT, "2FEP", "b2", 1, 0); 


@ selecting residuals to compute the CD and alpha  exponent @

x_res = (reshape(bres11,rows(ni),rows(y)))';

{t_res,n_res,cdx} = CDP2(x_res);
"CD test as in KMSS";cdx;

@ compute the alpha exponent CSD as in BKP @

a_size=0.1;   
x_resst=standx(x_res);
/********** Exponent of cross-sectional dependence ************/
{rej,a_res,v_f_res,rej,rej,ggg_res,s_res,rej}=atildeall(x_resst,a_size);
om_res=1.65*((1/t_res)*(v_f_res)+(4/n_res)*(n_res^(1-a_res)*s_res/(ggg_res-1)))^(1/2)/(2*ln(n_res)); /* 90% CI: 1.65, 95% CI: 1.96 */
/* ***************************** Output ******************************* */
"";"";
" Exponents of CSD (alpha) and 90% c.i. as in BKP ";
"a_res"; (a_res-om_res)~a_res~(a_res+om_res);
"";"";

@ excel output for CSD test@
ret = xlsWritem(cdx, "2FEP", "c10", 1, 0); 
ret = xlsWritem((a_res-om_res)~a_res~(a_res+om_res), "2FEP", "c12", 1, 0); 

@ estimates of 3DCCE_G @

{bp22,bmg22,vbp22,vbmg22,tbp22,tbmg22,bres22,bsep22,bsemg22}=index3mis(y,x,2,2,zeros(cols(data)-2,1),rows(ivar),ni,nj,5,7);

PROUT = Xname~bp22~bsep22~tbp22;

"BETA 3D CCEP_G ";"  regressors   bpool    s.e.     t-stat  ";
call printfm(PRout,0~1~1~1,"s"~8~8|"lf"~10~4|"lf"~10~4|"lf"~10~4); 
"";

@ output in excel 3DCCEP_G @
ret = xlsWritem(PROUT, "3CEEP_G", "b2", 1, 0); 
x_res = (reshape(bres22,rows(ni),rows(y)))';  /*  T by N */

{t,nn,cdx} = CDP2(x_res);
"CD test as in KMSS";cdx;

a_size=0.1;
x_resst=standx(x_res);

{t_res,n_res,cd_res}=CD(x_resst);
/********** Exponent of cross-sectional dependence ************/
{rej,a_res,v_f_res,rej,rej,ggg_res,s_res,rej}=atildeall(x_resst,a_size);
om_res=1.65*((1/t_res)*(v_f_res)+(4/n_res)*(n_res^(1-a_res)*s_res/(ggg_res-1)))^(1/2)/(2*ln(n_res)); /* 90% CI: 1.65, 95% CI: 1.96 */
/* ***************************** Output ******************************* */
"";"";
" Exponents of CSD (alpha) as in BKP and 90% c.i. ";
"a_res"; (a_res-om_res)~a_res~(a_res+om_res);
"";"";

@ excel output for CSD test@
ret = xlsWritem(cdx, "3CEEP_G", "c10", 1, 0); 
ret = xlsWritem((a_res-om_res)~a_res~(a_res+om_res), "3CEEP_G", "c12", 1, 0); 


@ estimates of 3DCCEP_L @

{bp32,bmg32,vbp32,vbmg32,tbp32,tbmg32,bres32,bsep32,bsemg32}=index3mis(y,x,3,2,zeros(cols(data)-2,1),rows(ivar),ni,nj,5,7);

PROUT = Xname~bp32~bsep32~tbp32;

"BETA 3D CCEP_L ";"  regressors   bpool    s.e.     t-stat  ";
call printfm(PRout,0~1~1~1,"s"~8~8|"lf"~10~4|"lf"~10~4|"lf"~10~4); 
"";

@ output in excel 3DCCEP_L @
ret = xlsWritem(PROUT, "3CEEP_L", "b2", 1, 0); 

x_res = (reshape(bres32,rows(ni),rows(y)))';  /*  T by N */

{t,nn,cdx} = CDP2(x_res);
"CD test as in KMSS";cdx;

a_size=0.1;
x_resst=standx(x_res);

{t_res,n_res,cd_res}=CD(x_resst);
/********** Exponent of cross-sectional dependence ************/
{rej,a_res,v_f_res,rej,rej,ggg_res,s_res,rej}=atildeall(x_resst,a_size);
om_res=1.65*((1/t_res)*(v_f_res)+(4/n_res)*(n_res^(1-a_res)*s_res/(ggg_res-1)))^(1/2)/(2*ln(n_res)); /* 90% CI: 1.65, 95% CI: 1.96 */
/* ***************************** Output ******************************* */
"";"";
"Exponents of CSD (alpha) as in BKP and 90% c.i. ";
"a_res"; (a_res-om_res)~a_res~(a_res+om_res);
"";"";

@ excel output for CSD test@
ret = xlsWritem(cdx, "3CEEP_L", "c10", 1, 0); 
ret = xlsWritem((a_res-om_res)~a_res~(a_res+om_res), "3CEEP_L", "c12", 1, 0); 


@ estimates of 3DCCEP_LG @

{bp42,bmg42,vbp42,vbmg42,tbp42,tbmg42,bres42,bsep42,bsemg42}=index3mis(y,x,4,2,zeros(cols(data)-2,1),rows(ivar),ni,nj,5,7);

PROUT = Xname~bp42~bsep42~tbp42;
"BETA 3D CCEP_GL ";"  regressors   bpool    s.e.     t-stat  ";
call printfm(PRout,0~1~1~1,"s"~8~8|"lf"~10~4|"lf"~10~4|"lf"~10~4); 
"";

@ output in excel 3DCCEP_LG @
ret = xlsWritem(PROUT, "3CEEP_LG", "b2", 1, 0); 

x_res = (reshape(bres42,rows(ni),rows(y)))';  /*  T by N */

{t,nn,cdx} = CDP2(x_res);
"CD test as in KMSS";cdx;

a_size=0.1;
x_resst=standx(x_res);

{t_res,n_res,cd_res}=CD(x_resst);
/********** Exponent of cross-sectional dependence ************/
{rej,a_res,v_f_res,rej,rej,ggg_res,s_res,rej}=atildeall(x_resst,a_size);
om_res=1.65*((1/t_res)*(v_f_res)+(4/n_res)*(n_res^(1-a_res)*s_res/(ggg_res-1)))^(1/2)/(2*ln(n_res)); /* 90% CI: 1.65, 95% CI: 1.96 */
/* ***************************** Output ******************************* */
"";"";
"Exponents of CSD (alpha) as in BKP and 90% c.i. ";
"a_res"; (a_res-om_res)~a_res~(a_res+om_res);
"";"";

@ excel output for CSD test@
ret = xlsWritem(cdx, "3CEEP_LG", "c10", 1, 0); 
ret = xlsWritem((a_res-om_res)~a_res~(a_res+om_res), "3CEEP_LG", "c12", 1, 0); 

end;
